@@ -1,5 +1,4 @@
 <?php
-
 // This file is part of Moodle - http://moodle.org/
 //
 // Moodle is free software: you can redistribute it and/or modify
@@ -15,7 +14,16 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-$plugin->version  = 2007073000;
-$plugin->requires = 2007101000;
+/**
+ * Version details for the outcomes report
+ *
+ * @package    gradereport_outcomes
+ * @copyright  1999 onwards Martin Dougiamas (http://dougiamas.com)
+ * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ */
 
-?>
+defined('MOODLE_INTERNAL') || die();
+
+$plugin->version   = 2022112800;        // The current plugin version (Date: YYYYMMDDXX).
+$plugin->requires  = 2022111800;        // Requires this Moodle version.
+$plugin->component = 'gradereport_outcomes'; // Full name of the plugin (used for diagnostics)
