@@ -27,12 +27,12 @@ use core_h5p\local\library\autoloader;
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @runTestsInSeparateProcesses
 */
-class generator_testcase extends \advanced_testcase {
+final class generator_test extends \advanced_testcase {
 
     /**
      * Tests set up.
      */
-    protected function setUp() {
+    protected function setUp(): void {
         parent::setUp();
 
         autoloader::register();
@@ -194,7 +194,7 @@ class generator_testcase extends \advanced_testcase {
      *
      * @return array
      */
-    public function generate_h5p_data_files_creation_provider(): array {
+    public static function generate_h5p_data_files_creation_provider(): array {
         return [
             'Do not create library related files on the filesystem' => [
                 false,
@@ -216,7 +216,9 @@ class generator_testcase extends \advanced_testcase {
 
         $generator = $this->getDataGenerator()->get_plugin_generator('core_h5p');
 
-        $data = $generator->create_library_record('Library', 'Lib', 1, 2, 3, 'Semantics example', '/regex11/');
+        $data = $generator->create_library_record(
+            'Library', 'Lib', 1, 2, 3, 'Semantics example', '/regex11/', 'http://tutorial.org/', 'http://example.org/'
+        );
         unset($data->id);
 
         $expected = (object) [
@@ -233,9 +235,12 @@ class generator_testcase extends \advanced_testcase {
             'droplibrarycss' => '',
             'semantics' => 'Semantics example',
             'addto' => '/regex11/',
+            'tutorial' => 'http://tutorial.org/',
+            'example' => 'http://example.org/',
             'coremajor' => null,
             'coreminor' => null,
             'metadatasettings' => null,
+            'enabled' => 1,
         ];
 
         $this->assertEquals($expected, $data);
@@ -271,7 +276,7 @@ class generator_testcase extends \advanced_testcase {
      *
      * @return array
      */
-    public function create_h5p_record_provider(): array {
+    public static function create_h5p_record_provider(): array {
         $createdjsoncontent = json_encode(
             array(
                 'text' => '<p>Created dummy text<\/p>\n',
@@ -392,7 +397,7 @@ class generator_testcase extends \advanced_testcase {
      *
      * @return array
      */
-    public function create_contents_libraries_record_provider(): array {
+    public static function create_contents_libraries_record_provider(): array {
         return [
             'Create h5p content library with set dependency type' => [
                 [
@@ -452,7 +457,7 @@ class generator_testcase extends \advanced_testcase {
      *
      * @return array
      */
-    public function create_library_dependency_record_provider(): array {
+    public static function create_library_dependency_record_provider(): array {
         return [
             'Create h5p library dependency with set dependency type' => [
                 [
@@ -518,7 +523,7 @@ class generator_testcase extends \advanced_testcase {
      *
      * @return array
      **/
-    public function create_content_file_provider(): array {
+    public static function create_content_file_provider(): array {
         return [
             'Create file in content with id 4' => [
                 [
