@@ -73,7 +73,7 @@ function(
     var getDefaultPagingBarTemplateContext = function() {
         return {
             showitemsperpageselector: false,
-            itemsperpage: 35,
+            itemsperpage: [{value: 35, active: true}],
             previous: true,
             next: true,
             activepagenumber: 1,
@@ -127,7 +127,7 @@ function(
         }
 
         var context = getDefaultPagingBarTemplateContext();
-        context.itemsperpage = itemsPerPage;
+        context.itemsperpage = buildItemsPerPagePagingBarContext(itemsPerPage);
         var numberOfPages = calculateNumberOfPages(numberOfItems, itemsPerPage);
 
         for (var i = 1; i <= numberOfPages; i++) {
@@ -166,9 +166,11 @@ function(
      * @return {int|array}
      */
     var buildItemsPerPagePagingBarContext = function(itemsPerPage) {
+        var context = [];
+
         if ($.isArray(itemsPerPage)) {
             // Convert the array into a format accepted by the template.
-            var context = itemsPerPage.map(function(num) {
+            context = itemsPerPage.map(function(num) {
                 if (typeof num === 'number') {
                     // If the item is just a plain number then convert it into
                     // an object with value and active keys.
@@ -190,11 +192,12 @@ function(
             if (!activeItems.length) {
                 context[0].active = true;
             }
-
-            return context;
         } else {
-            return itemsPerPage;
+            // Convert the integer into a format accepted by the template.
+            context = [{value: itemsPerPage, active: true}];
         }
+
+        return context;
     };
 
     /**
@@ -211,7 +214,8 @@ function(
 
         var context = getDefaultPagingBarTemplateContext();
         context.itemsperpage = buildItemsPerPagePagingBarContext(itemsPerPage);
-        context.showitemsperpageselector = $.isArray(itemsPerPage);
+        // Only display the items per page selector if there is more than one to choose from.
+        context.showitemsperpageselector = $.isArray(itemsPerPage) && itemsPerPage.length > 1;
 
         return context;
     };
@@ -343,6 +347,10 @@ function(
             context.pagingdropdown = buildPagingDropdownTemplateContext(itemsPerPage, config);
         } else {
             context.pagingbar = buildPagingBarTemplateContext(numberOfItems, itemsPerPage);
+            if (config.hasOwnProperty('showFirstLast') && config.showFirstLast) {
+                context.pagingbar.first = true;
+                context.pagingbar.last = true;
+            }
         }
 
         return context;
