@@ -26,6 +26,7 @@
 
 namespace mod_quiz\external;
 
+use core_question\local\bank\question_version_status;
 use externallib_advanced_testcase;
 use mod_quiz_external;
 use mod_quiz_display_options;
@@ -80,7 +81,7 @@ class testable_mod_quiz_external extends mod_quiz_external {
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  * @since      Moodle 3.1
  */
-class external_test extends externallib_advanced_testcase {
+final class external_test extends externallib_advanced_testcase {
 
     /**
      * Set up for every test
@@ -214,8 +215,8 @@ class external_test extends externallib_advanced_testcase {
 
         // Create what we expect to be returned when querying the two courses.
         // First for the student user.
-        $allusersfields = array('id', 'coursemodule', 'course', 'name', 'intro', 'introformat', 'introfiles', 'timeopen',
-                                'timeclose', 'grademethod', 'section', 'visible', 'groupmode', 'groupingid',
+        $allusersfields = array('id', 'coursemodule', 'course', 'name', 'intro', 'introformat', 'introfiles', 'lang',
+                                'timeopen', 'timeclose', 'grademethod', 'section', 'visible', 'groupmode', 'groupingid',
                                 'attempts', 'timelimit', 'grademethod', 'decimalpoints', 'questiondecimalpoints', 'sumgrades',
                                 'grade', 'preferredbehaviour', 'hasfeedback');
         $userswithaccessfields = array('attemptonlast', 'reviewattempt', 'reviewcorrectness', 'reviewmarks',
@@ -236,8 +237,10 @@ class external_test extends externallib_advanced_testcase {
         $quiz1->groupingid = 0;
         $quiz1->hasquestions = 0;
         $quiz1->hasfeedback = 0;
+        $quiz1->completionpass = 0;
         $quiz1->autosaveperiod = get_config('quiz', 'autosaveperiod');
         $quiz1->introfiles = [];
+        $quiz1->lang = '';
 
         $quiz2->coursemodule = $quiz2->cmid;
         $quiz2->introformat = 1;
@@ -247,8 +250,10 @@ class external_test extends externallib_advanced_testcase {
         $quiz2->groupingid = 0;
         $quiz2->hasquestions = 0;
         $quiz2->hasfeedback = 0;
+        $quiz2->completionpass = 0;
         $quiz2->autosaveperiod = get_config('quiz', 'autosaveperiod');
         $quiz2->introfiles = [];
+        $quiz2->lang = '';
 
         foreach (array_merge($allusersfields, $userswithaccessfields) as $field) {
             $expected1[$field] = $quiz1->{$field};
@@ -315,9 +320,9 @@ class external_test extends externallib_advanced_testcase {
         $result = \external_api::clean_returnvalue($returndescription, $result);
         $this->assertCount(2, $result['quizzes']);
         // We only see a limited set of fields.
-        $this->assertCount(4, $result['quizzes'][0]);
+        $this->assertCount(5, $result['quizzes'][0]);
         $this->assertEquals($quiz2->id, $result['quizzes'][0]['id']);
-        $this->assertEquals($quiz2->coursemodule, $result['quizzes'][0]['coursemodule']);
+        $this->assertEquals($quiz2->cmid, $result['quizzes'][0]['coursemodule']);
         $this->assertEquals($quiz2->course, $result['quizzes'][0]['course']);
         $this->assertEquals($quiz2->name, $result['quizzes'][0]['name']);
         $this->assertEquals($quiz2->course, $result['quizzes'][0]['course']);
@@ -1317,7 +1322,6 @@ class external_test extends externallib_advanced_testcase {
         // Now, get the summary.
         $result = mod_quiz_external::get_attempt_summary($attempt->id);
         $result = \external_api::clean_returnvalue(mod_quiz_external::get_attempt_summary_returns(), $result);
-        $this->assertDebuggingCalled(); // Expect $PAGE->set_url debugging.
 
         // Check it's marked as completed only the first one.
         $this->assertEquals('complete', $result['questions'][0]['state']);
@@ -1621,7 +1625,7 @@ class external_test extends externallib_advanced_testcase {
         $this->assertEventContextNotUsed($event);
         $this->assertNotEmpty($event->get_name());
 
-        // Now, force the quiz with QUIZ_NAVMETHOD_SEQ (sequencial) navigation method.
+        // Now, force the quiz with QUIZ_NAVMETHOD_SEQ (sequential) navigation method.
         $DB->set_field('quiz', 'navmethod', QUIZ_NAVMETHOD_SEQ, array('id' => $quiz->id));
         // Quiz requiring preflightdata.
         $DB->set_field('quiz', 'password', 'abcdef', array('id' => $quiz->id));
@@ -1939,11 +1943,15 @@ class external_test extends externallib_advanced_testcase {
         $question = $questiongenerator->create_question('shortanswer', null, array('category' => $cat->id));
         quiz_add_quiz_question($question->id, $quiz);
 
-        // Add new question types in the category (for the random one).
         $question = $questiongenerator->create_question('truefalse', null, array('category' => $cat->id));
-        $question = $questiongenerator->create_question('essay', null, array('category' => $cat->id));
+        quiz_add_quiz_question($question->id, $quiz);
 
-        quiz_add_random_questions($quiz, 0, $cat->id, 1, false);
+        $question = $questiongenerator->create_question('essay', null, array('category' => $cat->id));
+        quiz_add_quiz_question($question->id, $quiz);
+
+        $question = $questiongenerator->create_question('multichoice', null,
+                ['category' => $cat->id, 'status' => question_version_status::QUESTION_STATUS_DRAFT]);
+        quiz_add_quiz_question($question->id, $quiz);
 
         $this->setUser($this->student);
 
@@ -1951,7 +1959,7 @@ class external_test extends externallib_advanced_testcase {
         $result = \external_api::clean_returnvalue(mod_quiz_external::get_quiz_required_qtypes_returns(), $result);
 
         $expected = array(
-            'questiontypes' => ['essay', 'numerical', 'random', 'shortanswer', 'truefalse'],
+            'questiontypes' => ['essay', 'numerical', 'shortanswer', 'truefalse'],
             'warnings' => []
         );
 
@@ -1959,6 +1967,57 @@ class external_test extends externallib_advanced_testcase {
 
     }
 
+    /**
+     * Test get_quiz_required_qtypes for quiz with random questions
+     */
+    public function test_get_quiz_required_qtypes_random() {
+        $this->setAdminUser();
+
+        // Create a new quiz.
+        $quizgenerator = $this->getDataGenerator()->get_plugin_generator('mod_quiz');
+        $quiz = $quizgenerator->create_instance(['course' => $this->course->id]);
+
+        // Create some questions.
+        $questiongenerator = $this->getDataGenerator()->get_plugin_generator('core_question');
+
+        $cat = $questiongenerator->create_question_category();
+        $anothercat = $questiongenerator->create_question_category();
+
+        $question = $questiongenerator->create_question('numerical', null, ['category' => $cat->id]);
+        $question = $questiongenerator->create_question('shortanswer', null, ['category' => $cat->id]);
+        $question = $questiongenerator->create_question('truefalse', null, ['category' => $cat->id]);
+        // Question in a different category.
+        $question = $questiongenerator->create_question('essay', null, ['category' => $anothercat->id]);
+
+        // Add a couple of random questions from the same category.
+        quiz_add_random_questions($quiz, 0, $cat->id, 1, false);
+        quiz_add_random_questions($quiz, 0, $cat->id, 1, false);
+
+        $this->setUser($this->student);
+
+        $result = mod_quiz_external::get_quiz_required_qtypes($quiz->id);
+        $result = \external_api::clean_returnvalue(mod_quiz_external::get_quiz_required_qtypes_returns(), $result);
+
+        $expected = ['numerical', 'shortanswer', 'truefalse'];
+        ksort($result['questiontypes']);
+
+        $this->assertEquals($expected, $result['questiontypes']);
+
+        // Add more questions to the quiz, this time from the other category.
+        $this->setAdminUser();
+        quiz_add_random_questions($quiz, 0, $anothercat->id, 1, false);
+
+        $this->setUser($this->student);
+        $result = mod_quiz_external::get_quiz_required_qtypes($quiz->id);
+        $result = \external_api::clean_returnvalue(mod_quiz_external::get_quiz_required_qtypes_returns(), $result);
+
+        // The new question from the new category is returned as a potential random question for the quiz.
+        $expected = ['essay', 'numerical', 'shortanswer', 'truefalse'];
+        ksort($result['questiontypes']);
+
+        $this->assertEquals($expected, $result['questiontypes']);
+    }
+
     /**
      * Test that a sequential navigation quiz is not allowing to see questions in advance except if reviewing
      */
@@ -2034,7 +2093,7 @@ class external_test extends externallib_advanced_testcase {
      *
      * @return quiz
      */
-    private function prepare_sequential_quiz() {
+    private function prepare_sequential_quiz(): quiz {
         // Create a new quiz with 5 questions and one attempt started.
         // Create a new quiz with attempts.
         $quizgenerator = $this->getDataGenerator()->get_plugin_generator('mod_quiz');
@@ -2075,7 +2134,7 @@ class external_test extends externallib_advanced_testcase {
      * @return quiz_attempt
      * @throws \moodle_exception
      */
-    private function create_quiz_attempt_object($quizobj, $userid = null, $ispreview = false) {
+    private function create_quiz_attempt_object(quiz $quizobj, ?int $userid = null, ?bool $ispreview = false): quiz_attempt {
         global $USER;
         $timenow = time();
         // Now, do one attempt.
