@@ -18,6 +18,7 @@ namespace qbank_statistics;
 
 use core_question\statistics\questions\all_calculated_for_qubaid_condition;
 use quiz;
+use quiz_statistics\tests\statistics_helper;
 use question_engine;
 use quiz_attempt;
 
@@ -29,7 +30,7 @@ use quiz_attempt;
  * @author     Nathan Nguyen <nathannguyen@catalyst-au.net>
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class helper_test extends \advanced_testcase {
+final class helper_test extends \advanced_testcase {
 
     /**
      * Test quizzes that contain a specified question.
@@ -193,7 +194,7 @@ class helper_test extends \advanced_testcase {
      *
      * @return array
      */
-    private function generate_attempt_answers(array $correctanswerflags): array {
+    private static function generate_attempt_answers(array $correctanswerflags): array {
         $attempt = [];
         for ($i = 1; $i <= 4; $i++) {
             if (isset($correctanswerflags) && $correctanswerflags[$i - 1] == 1) {
@@ -228,8 +229,7 @@ class helper_test extends \advanced_testcase {
 
         // Calculate the statistics.
         $this->expectOutputRegex('~.*Calculations completed.*~');
-        $statisticstask = new \quiz_statistics\task\recalculate();
-        $statisticstask->execute();
+        statistics_helper::run_pending_recalculation_tasks();
 
         return [$quiz1, $quiz2, $questions];
     }
@@ -266,31 +266,31 @@ class helper_test extends \advanced_testcase {
      *
      * @return \Generator
      */
-    public function load_question_facility_provider(): \Generator {
+    public static function load_question_facility_provider(): \Generator {
         yield 'Facility case 1' => [
             'Quiz 1 attempts' => [
-                $this->generate_attempt_answers([1, 0, 0, 0]),
+                self::generate_attempt_answers([1, 0, 0, 0]),
             ],
             'Expected quiz 1 facilities' => ['100.00%', '0.00%', '0.00%', '0.00%'],
             'Quiz 2 attempts' => [
-                $this->generate_attempt_answers([1, 0, 0, 0]),
-                $this->generate_attempt_answers([1, 1, 0, 0]),
+                self::generate_attempt_answers([1, 0, 0, 0]),
+                self::generate_attempt_answers([1, 1, 0, 0]),
             ],
             'Expected quiz 2 facilities' => ['100.00%', '50.00%', '0.00%', '0.00%'],
             'Expected average facilities' => ['100.00%', '25.00%', '0.00%', '0.00%'],
         ];
         yield 'Facility case 2' => [
             'Quiz 1 attempts' => [
-                $this->generate_attempt_answers([1, 0, 0, 0]),
-                $this->generate_attempt_answers([1, 1, 0, 0]),
-                $this->generate_attempt_answers([1, 1, 1, 0]),
+                self::generate_attempt_answers([1, 0, 0, 0]),
+                self::generate_attempt_answers([1, 1, 0, 0]),
+                self::generate_attempt_answers([1, 1, 1, 0]),
             ],
             'Expected quiz 1 facilities' => ['100.00%', '66.67%', '33.33%', '0.00%'],
             'Quiz 2 attempts' => [
-                $this->generate_attempt_answers([1, 0, 0, 0]),
-                $this->generate_attempt_answers([1, 1, 0, 0]),
-                $this->generate_attempt_answers([1, 1, 1, 0]),
-                $this->generate_attempt_answers([1, 1, 1, 1]),
+                self::generate_attempt_answers([1, 0, 0, 0]),
+                self::generate_attempt_answers([1, 1, 0, 0]),
+                self::generate_attempt_answers([1, 1, 1, 0]),
+                self::generate_attempt_answers([1, 1, 1, 1]),
             ],
             'Expected quiz 2 facilities' => ['100.00%', '75.00%', '50.00%', '25.00%'],
             'Expected average facilities' => ['100.00%', '70.83%', '41.67%', '12.50%'],
@@ -359,20 +359,20 @@ class helper_test extends \advanced_testcase {
      * Data provider for {@see test_load_question_discriminative_efficiency()}.
      * @return \Generator
      */
-    public function load_question_discriminative_efficiency_provider(): \Generator {
+    public static function load_question_discriminative_efficiency_provider(): \Generator {
         yield 'Discriminative efficiency' => [
             'Quiz 1 attempts' => [
-                $this->generate_attempt_answers([1, 0, 0, 0]),
-                $this->generate_attempt_answers([1, 1, 0, 0]),
-                $this->generate_attempt_answers([1, 0, 1, 0]),
-                $this->generate_attempt_answers([1, 1, 1, 1]),
+                self::generate_attempt_answers([1, 0, 0, 0]),
+                self::generate_attempt_answers([1, 1, 0, 0]),
+                self::generate_attempt_answers([1, 0, 1, 0]),
+                self::generate_attempt_answers([1, 1, 1, 1]),
             ],
             'Expected quiz 1 discriminative efficiency' => ['N/A', '33.33%', '33.33%', '100.00%'],
             'Quiz 2 attempts' => [
-                $this->generate_attempt_answers([1, 1, 1, 1]),
-                $this->generate_attempt_answers([0, 0, 0, 0]),
-                $this->generate_attempt_answers([1, 0, 0, 1]),
-                $this->generate_attempt_answers([0, 1, 1, 0]),
+                self::generate_attempt_answers([1, 1, 1, 1]),
+                self::generate_attempt_answers([0, 0, 0, 0]),
+                self::generate_attempt_answers([1, 0, 0, 1]),
+                self::generate_attempt_answers([0, 1, 1, 0]),
             ],
             'Expected quiz 2 discriminative efficiency' => ['50.00%', '50.00%', '50.00%', '50.00%'],
             'Expected average discriminative efficiency' => ['50.00%', '41.67%', '41.67%', '75.00%'],
@@ -465,20 +465,20 @@ class helper_test extends \advanced_testcase {
      * Data provider for {@see test_load_question_discrimination_index()}.
      * @return \Generator
      */
-    public function load_question_discrimination_index_provider(): \Generator {
+    public static function load_question_discrimination_index_provider(): \Generator {
         yield 'Discrimination Index' => [
             'Quiz 1 attempts' => [
-                $this->generate_attempt_answers([1, 0, 0, 0]),
-                $this->generate_attempt_answers([1, 1, 0, 0]),
-                $this->generate_attempt_answers([1, 0, 1, 0]),
-                $this->generate_attempt_answers([1, 1, 1, 1]),
+                self::generate_attempt_answers([1, 0, 0, 0]),
+                self::generate_attempt_answers([1, 1, 0, 0]),
+                self::generate_attempt_answers([1, 0, 1, 0]),
+                self::generate_attempt_answers([1, 1, 1, 1]),
             ],
             'Expected quiz 1 Discrimination Index' => ['N/A', '30.15%', '30.15%', '81.65%'],
             'Quiz 2 attempts' => [
-                $this->generate_attempt_answers([1, 1, 1, 1]),
-                $this->generate_attempt_answers([0, 0, 0, 0]),
-                $this->generate_attempt_answers([1, 0, 0, 1]),
-                $this->generate_attempt_answers([0, 1, 1, 0]),
+                self::generate_attempt_answers([1, 1, 1, 1]),
+                self::generate_attempt_answers([0, 0, 0, 0]),
+                self::generate_attempt_answers([1, 0, 0, 1]),
+                self::generate_attempt_answers([0, 1, 1, 0]),
             ],
             'Expected quiz 2 discrimination Index' => ['44.72%', '44.72%', '44.72%', '44.72%'],
             'Expected average discrimination Index' => ['44.72%', '37.44%', '37.44%', '63.19%'],
